/*
    Copyright (C) 2003 by Stephan Linz <linz@li-pro.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Bosten, MA 02111-1307 USA
*/

/* $Id: $ */

/*
   avrctrl/delay.h - loops for small accurate delays (like avr/delay.h)
 */

#ifndef _AVRCTRL_DELAY_H_
#define _AVRCTRL_DELAY_H_ 1

#if defined(HAVE_CONFIG_H) && defined(AVRCTRL_LIB_SRC)
#include <config.h>
#else
#include <avrctrl/config.h>
#endif

#ifndef __ASSEMBLER__

/** \defgroup avrctrl_delay Unterbrechungen / Verzgerungen (ungenau)
    \code #include <avrctrl/delay.h> \endcode

    Diese Headerdatei deklariert ...

    \todo macros to allow specifying delays directly in microseconds
    (with MCU clock frequency defined by the user).  With constant
    delays, all floating point math would be done at compile time. */

/** \name Deklarationen mit Abhngigkeit vom Systemtakt */

/*@{*/

#ifdef __cplusplus
extern "C" {
#endif

/** \fn void delay_ms(unsigned int msec)
    \ingroup avrctrl_delay
    \brief Unterbricht die Ausfhrung fr einen Intervall von Millisekunden.

    Die Funktion delay_ms() unterbricht die Ausfhrung der aufrufenden Instanz
    fr msec Millisekunden. Die Unterbrechung kann durch Systemaktivitten,
    z.B. Interrupts, oder durch die Zeit, die zum Bearbeiten des Aufrufs
    verwendet wird, verlngert werden.

    Der regulre Jitter betrgt etwa 0,29 % der Gesamtverzgerung. Die
    Funktion delay_ms() benutzt den WDR Aufruf, um ein eventuelles Reset durch
    den Watchdog zu vermeiden.

    \returns Die Funktion delay_ms() besitzt keinen Rckgabewert. */
extern void delay_ms(unsigned int msec);

/** \fn void delay_us(unsigned int usec)
    \ingroup avrctrl_delay
    \brief Unterbricht die Ausfhrung fr einen Intervall von Mikrosekunden.

    Die Funktion delay_us() unterbricht die Ausfhrung der aufrufenden Instanz
    fr usec Mikrosekunden. Die Unterbrechung kann durch Systemaktivitten,
    z.B. Interrupts, oder durch die Zeit, die zum Bearbeiten des Aufrufs
    verwendet wird, verlngert werden.

    \returns Die Funktion delay_us() besitzt keinen Rckgabewert. */
extern void delay_us(unsigned int usec);

#ifdef __cplusplus
}
#endif

/*@}*/

/** \name Deklarationen unabhngig vom Systemtakt */

/*@{*/

/** \def delay_9T()
    \ingroup avrctrl_delay
    Unterbricht die Ausfhrung fr einen Intervall von 9 Systemtakten. */
#define delay_9T()					\
	asm volatile(	"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
		::)

/** \def delay_8T()
    \ingroup avrctrl_delay
    Unterbricht die Ausfhrung fr einen Intervall von 8 Systemtakten. */
#define delay_8T()					\
	asm volatile(	"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
		::)

/** \def delay_7T()
    \ingroup avrctrl_delay
    Unterbricht die Ausfhrung fr einen Intervall von 7 Systemtakten. */
#define delay_7T()					\
	asm volatile(	"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
		::)

/** \def delay_6T()
    \ingroup avrctrl_delay
    Unterbricht die Ausfhrung fr einen Intervall von 6 Systemtakten. */
#define delay_6T()					\
	asm volatile(	"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
		::)

/** \def delay_5T()
    \ingroup avrctrl_delay
    Unterbricht die Ausfhrung fr einen Intervall von 5 Systemtakten. */
#define delay_5T()					\
	asm volatile(	"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
		::)

/** \def delay_4T()
    \ingroup avrctrl_delay
    Unterbricht die Ausfhrung fr einen Intervall von 4 Systemtakten. */
#define delay_4T()					\
	asm volatile(	"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
		::)

/** \def delay_3T()
    \ingroup avrctrl_delay
    Unterbricht die Ausfhrung fr einen Intervall von 3 Systemtakten. */
#define delay_3T()					\
	asm volatile(	"nop\n\t"			\
			"nop\n\t"			\
			"nop\n\t"			\
		::)

/** \def delay_2T()
    \ingroup avrctrl_delay
    Unterbricht die Ausfhrung fr einen Intervall von 2 Systemtakten. */
#define delay_2T()					\
	asm volatile(	"nop\n\t"			\
			"nop\n\t"			\
		::)

/** \def delay_1T()
    \ingroup avrctrl_delay
    Unterbricht die Ausfhrung fr einen Intervall von 1 Systemtakt. */
#define delay_1T()					\
	asm volatile(	"nop\n\t"			\
		::)

/*@}*/

#endif /* __ASSEMBLER__ */

#endif /* _AVRCTRL_DELAY_H_ */
